#ifndef GPXI31128_HEADER
#define GPXI31128_HEADER

/*-------------------------------------------------------------------------*/
/* Defines                                                                 */
/*-------------------------------------------------------------------------*/
#define K_DEV_MAX        12
#define K_DLL_VERSION   100     // Version 1.00

// XILINX command structure
#define K_HEADER_XILINX_INSTRUCTION_GLOBAL      0    // start byte
#define K_HEADER_XILINX_CARD_NUMBER             1    // device (1 ... K_DEV_MAX)
#define K_HEADER_XILINX_PORT_NUMBER             2    // not used
#define K_HEADER_XILINX_INSTRUCTION             3    // command
#define K_HEADER_XILINX_FIRST_DATA              4    
#define K_HEADER_XILINX_LENGTH                  4     

#define K_READ                      0
#define K_WRITE                     1

#define K_HEADER_XILINX_START_BYTE  0x48

// XILINX commands
#define K_XILINX_CMD_PO_RESET               0x10
#define K_XILINX_CMD_EXT_CLOCK              0x11
#define K_XILINX_CMD_REG_WRITE              0x12

#define K_XILINX_CMD_RELAY_CURRENT_STATE    0x20  
#define K_XILINX_CMD_RELAY_TARGET_STATE     0x21
#define K_XILINX_CMD_RELAY_TARGET_MASK      0x22
#define K_XILINX_CMD_RELAY_UPDATE           0x23
#define K_XILINX_CMD_RELAY_EXTENDED_INFO    0x24

/*-------------------------------------------------------------------------*/
/* Typedefs                                                                */
/*-------------------------------------------------------------------------*/
typedef struct
	 {
	 unsigned long Version;
	 unsigned long DevCount;
  unsigned long SW_XILINX[K_DEV_MAX];
  unsigned long BusIndexes[K_DEV_MAX];
	 unsigned long DevIndexes[K_DEV_MAX];
  unsigned long GeoAddress[K_DEV_MAX];
  unsigned long SerialNumber[K_DEV_MAX];
  unsigned long Revision[K_DEV_MAX];
	 } GPxi31128_StructDriverInfo;

typedef struct
{ 
  unsigned long Relay_1_32;
  unsigned long Relay_33_64;
  unsigned long Relay_65_96;
  unsigned long Relay_97_128;
} GPxi31128_StructReadRelayData;

typedef struct
{
  unsigned long Mode;
  unsigned long Register;
  unsigned long Value;
} GPxi31128_StructXilinxReadWrite;

/*-------------------------------------------------------------------------*/
/* Errorcodes                                                              */
/*-------------------------------------------------------------------------*/
// 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  Sev:      is the severity code
//               00 - Success
//               01 - Informational
//               10 - Warning
//               11 - Error
//  C:        is the Customer code flag
//  R:        is a reserved bit
//  Facility: is the facility code
//  Code:     is the facility's status code

// error codes (all sources)
#define K_NO_ERROR                               0x00000000
// error driver
#define K_UNKNOWN_IOCTRL                         0xE0000001
#define K_NOTSUPPORTED_DEVICE                    0xE0000002
#define K_XILINX_DOWNLOADERROR                   0xE0000003
#define K_XILINX_WRITE_DATA_HEADER               0xE0000004

#define K_STATUS_EEPROM_NO_ACK                   0xE000000A
#define K_STATUS_EEPROM_NO_START_CONDITION       0xE000000B
#define K_STATUS_EEPROM_UNDEFINED                0xE000000C

#define K_UNDEFINED_ERROR                        0xE00000F0
// error library
#define K_INVALID_HANDLE                         0xE0000100
#define K_ERR_DLINVALIDID                        0xE0000101
#define K_ERR_DLINVALIDVERSION                   0xE0000102
#define K_ERR_DLSIZE                             0xE0000103
#define K_ERR_DLFILE                             0xE0000104
#define K_NO_CARD_PRESENT                        0xE0000105


/*-------------------------------------------------------------------------*/
/* Functions                                                               */
/*-------------------------------------------------------------------------*/
int GPxi31128_GetDriverInfo(GPxi31128_StructDriverInfo* pDriverInfo);
int GPxi31128_DLL_Version(unsigned long *pVersion);
int GPxi31128_XilinxDownload(unsigned long Device, char* Path);
int GPxi31128_ReadWriteSpezial(unsigned char *Data, unsigned long *Length);
int GPxi31128_XilinxWriteData32(unsigned long *Data, unsigned long *Length);
int GPxi31128_SetRelay(unsigned long Device,
                       unsigned long Relay_1_32,
                       unsigned long Relay_33_64,
                       unsigned long Relay_65_96,
                       unsigned long Relay_97_128);
int GPxi31128_SetRelayMask(unsigned long Device,
                           unsigned long Relay_1_32,
                           unsigned long Relay_33_64,
                           unsigned long Relay_65_96,
                           unsigned long Relay_97_128,
                           unsigned long RelayMask_1_32,
                           unsigned long RelayMask_33_64,
                           unsigned long RelayMask_65_96,
                           unsigned long RelayMask_97_128);
int GPxi31128_ReadCurrentRelayValue(unsigned long                  Device,
                                    GPxi31128_StructReadRelayData *RelayValue);
int GPxi31128_ReadTargetRelayValue(unsigned long                  Device,
                                   GPxi31128_StructReadRelayData *RelayValue);
int GPxi31128_UpdateRelay(unsigned long Device);



#endif
